#!/usr/bin/python

# RSS Mix Tape
# Copyright 2003, David "Novalis" Turner <novalis@fsf.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from qt import *
from qttable import *
from khtml import *
from kdecore import *
from kparts import *

import time

from MixTape import MixTape
from FeedEditorImpl import FeedEditorImpl
from AboutBox import AboutBox
from FeedLoader import feedLoader
from FeedDB2 import feedDB
from RSSGenerator import RSSGenerator
from ConfigEditorImpl import ConfigEditorImpl

from Config import config

from threading import Lock

class MixTapeImpl (MixTape):
    def __init__ (self, *args):
        MixTape.__init__ (self, *args)
        header = self.postTable.horizontalHeader ();
        header.setLabel (0, "Posted to")
        header.setLabel (1, "From")
        header.setLabel (2, "Date")
        header.setLabel (3, "Title")

        self.postTable.setSelectionMode (QTable.SingleRow)

        
        self.browserPart = KHTMLPart (self.stack,"browser")
        self.browser = self.browserPart.view ()
        
        self.connect (self.browserPart,
                      SIGNAL ("onURL(const QString&)"), self.url.setText)

        self.connect (self.browserPart.browserExtension(),
                      SIGNAL ("openURLRequestDelayed(const KURL&, const KParts::URLArgs&)"), self.openURL)
        
        self.stack.removeWidget (self.page)
        self.stack.addWidget(self.browser)
        self.stack.raiseWidget(self.browser)

        self.postQueue = []
        self.threadLock = Lock ()
        self.initFeeds ()
     
        self.postTable.setLeftMargin (0)
        self.postTable.verticalHeader ().hide ()

        self.startTimer (1000)
        self.lastRow = -1

    def openURL (self, url, args):
        if self.currentPage:
            self.history.append (self.currentPage)
        self.currentPage = ([KURL (url), KParts.URLArgs (args)])
        self.browserPart.browserExtension ().setURLArgs (args)
        self.browserPart.openURL (url)

    def initFeeds (self):
        db = feedDB ()
        db.addListener (self.addFeed)
        for feed in db.feedList ():
            feed.open ()
            feed.setListener (self.addPostQueued)
            feed.readPosts ()
            feed.sync ()
            db.store (feed)

    def update_all_feeds (self):
        feedDB ().updateAll ()

    def addFeed (self, feed):
        feed.setListener (self.addPostQueued)

    def generate (self):
        #save current text
        row = self.postTable.currentRow ()
        cat = self.postTable.item (row, 0)
        cat.post ().comment = unicode (self.comment.text ())

        #generate
        r = RSSGenerator ()
        r.generate ()

    def edit_feeds (self):
       self.feedEditor = FeedEditorImpl (self)
       self.feedEditor.show ()
        
    def closeEvent (self, e):
        db = feedDB ()
        feedDB ().close ()
        return MixTape.closeEvent (self, e)

    def addPostQueued (self, post):
        """This is called from other threads to store posts for
        addition by the main GUI thread.  Only the main GUI thread may
        make xlib calls.  """
        self.threadLock.acquire ()      
        self.postQueue.append (post)
        self.threadLock.release ()
        
    def addPost (self, post):
        """Call this when a new post is added to the system"""
        if post.deleted:
            return #don't add it to the GUI
        
        table = self.postTable
        
        #extend the table by one row
        rowNum = table.numRows ()
        table.setNumRows (rowNum + 1)

        #add the category selector
        cat = CatDropDown (table, post)
        table.setItem (rowNum, 0, cat)

        #the feed name
        fromItem = QTableItem (table, QTableItem.Never, post.feed)
        table.setItem (rowNum, 1, fromItem)

        #date
        strdate = time.asctime (time.localtime (post.date))
        dateItem = QTableItem (table, QTableItem.Never, strdate)
        table.setItem (rowNum, 2, dateItem)

        #title
        titleItem = QTableItem (table, QTableItem.Never, post.title)
        table.setItem (rowNum, 3, titleItem)

    def delete_post (self):
        table = self.postTable
        row = table.currentRow ()
        if table.numRows () == 0 or row >= table.numRows () or row < 0:
            return
        
        #save commentary
        post = table.item (row, 0).post ()
        post.comment = unicode (self.comment.text ())

        post.delete ()

        self.lastRow = -1
        table.removeRow (row)
        self.lastRow = -1
        
        #if it's the last item, go back to the previous post
        if row == table.numRows () - 1:
            table.setCurrentCell (row - 1, 0)
        else:
            table.setCurrentCell (row, 0)

        self.view_post ()
        
    def view_post (self):
        table = self.postTable
        #save commentary, QuickTopic status
        if self.lastRow != -1:
            cat = table.item (self.lastRow, 0)
            cat.post ().comment = unicode (self.comment.text ())
            cat.post ().createQT = self.createQTCheck.isChecked ()
                
        #now put in new commentary & post
        row = table.currentRow ()
        if table.numRows () == 0 or row < 0 or row >= table.numRows ():
            return
        cat = table.item (row, 0)
        post = cat.post ()

        self.createQTCheck.setChecked (post.createQT)
        enabled = config ().get ("quicktopic", "email") and \
                  post.enableCreateQuickTopic ()
        self.createQTCheck.setEnabled (bool (enabled))
                    
        self.history = [["begin", ""]]
        self.currentPage = None
        
        self.browserPart.begin ()
        if post.link:
            self.browserPart.write ('<a href="' + post.link + '">Link</a><br>')
        self.browserPart.write (post.text)
        self.browserPart.end ()
        
        self.comment.setText (post.comment)
        self.lastRow = row

    def value_changed (self):
        table = self.postTable
        row = table.currentRow ()
        cat = table.item (row, 0)
        cat.updatePost ()

    def timerEvent (self, event):
        self.threadLock.acquire ()      
        for post in self.postQueue:
            self.addPost (post)
        self.postQueue = []
        self.threadLock.release ()
        for i in range (0, 4):
            self.postTable.adjustColumn (i)

        MixTape.timerEvent (self, event)

    def configure (self):
        cei = ConfigEditorImpl ()
        cei.exec_loop ()

        #regenerate the dropdowns for the categoriess
        for row in range (0, self.postTable.numRows ()):
            cat = self.postTable.item (row, 0)
            cat.reInit ()


    def about (self):
        a = AboutBox (self, None, 1)
        a.show ()

    def back (self):
        if self.history:
            url, args = self.history.pop ()
            if url == "begin":
                #this is the first item in history, thus the post itself
                row = self.postTable.currentRow ()
                cat = self.postTable.item (row, 0)
                post = cat.post ()
                self.browserPart.begin ()
                if post.link:
                    self.browserPart.write ('<a href="' + post.link + '">Link</a><br>')
                self.browserPart.write (post.text)
                self.browserPart.end ()
        
                self.history.append ([url, args])
            else:
                self.browserPart.browserExtension ().setURLArgs (args)
                self.browserPart.openURL (url) 

        self.currentPage = [url, args]

class CatDropDown (QComboTableItem):
    def __init__ (self, parent, post):
        cats = QStringList.fromStrList (["not posted"] + config ().getCategories ())
        QComboTableItem.__init__ (self, parent, cats)
        self._post = post
        self.setCurrentItem (post.posted)

    def post (self):
        return self._post

    def set (self, i):
        self.setCurrentItem (i)
        self._post.posted = str (i)

    def updatePost (self):
        self._post.posted = str (self.currentText ())

    def reInit (self):
        """When the categories have been edited, regenerate the dropdown"""
        cats = QStringList.fromStrList (["not posted"] + config ().getCategories ())
        self.setStringList (cats)
        #fixme:
        self.setCurrentItem (self._post.posted)
